[![Storybook](https://cdn.jsdelivr.net/gh/storybookjs/brand@master/badge/badge-storybook.svg)](https://storybook.js.org/)

# Roadmunk's Design System Component Library

## Getting Started

You want to use an awesome component library made by Roadmunkers, for Roadmunkers, you say? Well, you've come to the right place!

### Requirements

- An application which is using React
- A package manager such as NPM or Yarn

### Install the component library

_(A) Option with NPM:_

```bash
npm install @roadmunk/ds-components react react-dom styled-components styled-system
```

_(B) Option with Yarn:_

```bash
yarn add @roadmunk/ds-components react react-dom styled-components styled-system
```

### Add the `<AppProvider>` component to your application

**Note:** It is recommended that this be a top level scope within your application such as a primary layout or router. In most cases, there should only be one instance of `AppProvider` in an application.

For example:

```js
import React from 'react';

export const App = () => (
  <div className="App">
    <h1>Hello World</h1>
    <h2>Start editing to see some magic happen!</h2>
  </div>
);
```

becomes:

```js
import React from 'react';
import { AppProvider, theme } from '@roadmunk/ds-components';

export const App = () => (
  <AppProvider theme={theme}>
    <div className="App">
      <h1>Hello World</h1>
      <h2>Start editing to see some magic happen!</h2>
    </div>
  </AppProvider>
);
```

**Note:** we also add the `theme` prop. This provides the theme styles from the design system to all design system components, as well as to any custom [styled components](https://styled-components.com/) you may want to write.

## Component Usage

Visit the [DS Storybook](https://rm-ds-components.netlify.app/) for documentation on components in the design system.

## Contributing

Documentation for contributors lives in [CONTRIBUTING.md](./CONTRIBUTING.md).
