/// <reference types="react" />
import { ReactNode } from '@types/react';
import { StyledComponent } from 'styled-components';

interface AppProviderProps {
  theme?: any;
  children: React.ReactNode;
}

declare const AppProvider: ({ theme, children }: AppProviderProps) => JSX.Element;

interface BorderProps {
  border?: any;
  borderWidth?: any;
  borderStyle?: any;
  borderColor?: any;
  borderRadius?: any;
  borderTop?: any;
  borderTopWidth?: any;
  borderTopStyle?: any;
  borderTopColor?: any;
  borderTopLeftRadius?: any;
  borderTopRightRadius?: any;
  borderRight?: any;
  borderRightWidth?: any;
  borderRightStyle?: any;
  borderRightColor?: any;
  borderBottom?: any;
  borderBottomWidth?: any;
  borderBottomStyle?: any;
  borderBottomColor?: any;
  borderBottomLeftRadius?: any;
  borderBottomRightRadius?: any;
  borderLeft?: any;
  borderLeftWidth?: any;
  borderLeftStyle?: any;
  borderLeftColor?: any;
  borderX?: any;
  borderY?: any;
}

interface SpaceProps {
  m?: any;
  margin?: any;
  mt?: any;
  marginTop?: any;
  mr?: any;
  marginRight?: any;
  mb?: any;
  marginBottom?: any;
  ml?: any;
  marginLeft?: any;
  mx?: any;
  marginX?: any;
  my?: any;
  marginY?: any;
  p?: any;
  padding?: any;
  pt?: any;
  paddingTop?: any;
  pr?: any;
  paddingRight?: any;
  pb?: any;
  paddingBottom?: any;
  pl?: any;
  paddingLeft?: any;
  px?: any;
  paddingX?: any;
  py?: any;
  paddingY?: any;
}

interface ColorProps {
  color?: any;
  bg?: any;
  backgroundColor?: any;
  opacity?: any;
}

interface TypographyProps {
  fontFamily?: any;
  fontSize?: any;
  fontWeight?: any;
  lineHeight?: any;
  letterSpacing?: any;
  textAlign?: any;
  fontStyle?: any;
}

interface TransitionProps {
  transitionProperty?: any;
  transition?: any;
  timing?: any;
  easing?: any;
}

interface LayoutProps {
  width?: any;
  height?: any;
  minWidth?: any;
  maxWidth?: any;
  minHeight?: any;
  maxHeight?: any;
  size?: any;
  display?: any;
  verticalAlign?: any;
  overflow?: any;
  overflowX?: any;
  overflowY?: any;
}

interface FlexboxProps {
  alignItems?: any;
  alignContent?: any;
  justifyItems?: any;
  justifyContent?: any;
  flexWrap?: any;
  flexDirection?: any;
  flex?: any;
  flexGrow?: any;
  flexShrink?: any;
  flexBasis?: any;
  justifySelf?: any;
  alignSelf?: any;
  order?: any;
}

interface GridProps {
  gridGap?: any;
  gridRowGap?: any;
  gridColumnGap?: any;
  gridColumn?: any;
  gridRow?: any;
  gridArea?: any;
  gridAutoFlow?: any;
  gridAutoRows?: any;
  gridAutoColumns?: any;
  gridTemplateRows?: any;
  gridTemplateColumns?: any;
  gridTemplateAreas?: any;
}

interface PositionProps {
  position?: any;
  zIndex?: any;
  top?: any;
  right?: any;
  bottom?: any;
  left?: any;
}

interface ShadowProps {
  boxShadow?: any;
}

interface OutlineProps {
  outline?: any;
}

interface AdditionalPropsInterface {
  [key: string]: any;
}

interface ThemeInterface {
  borders: any;
  borderStyles: any;
  borderWidths: any;
  breakpoints: any;
  breakpointsWithMeta: any;
  buttonSizes: any;
  buttonVariants: any;
  colors: any;
  easings: any;
  fonts: any;
  fontSizes: any;
  fontWeights: any;
  letterSpacings: any;
  lineHeights: any;
  radii: any;
  semanticTextElements: any;
  shadows: any;
  space: any;
  spinnerSizes: any;
  textVariants: any;
  timings: any;
  transitions: any;
}

interface OtherProps {
  children?: ReactNode;
}

type BoxProps = OtherProps &
  BorderProps &
  SpaceProps &
  ColorProps &
  TypographyProps &
  TransitionProps &
  LayoutProps &
  FlexboxProps &
  GridProps &
  PositionProps &
  ShadowProps &
  OutlineProps;

declare const Box: StyledComponent<"div", any, OtherProps & BorderProps & SpaceProps & ColorProps & TypographyProps & TransitionProps & LayoutProps & FlexboxProps & GridProps & PositionProps & ShadowProps & OutlineProps & AdditionalPropsInterface, never>;

type TextSemanticElement =
  | 'h1'
  | 'h2'
  | 'h3'
  | 'h4'
  | 'h5'
  | 'p'
  | 'span'
  | 'label';

type TextStyleVariant = 'success' | 'error' | 'understated';

type TextVariant =
  | 'headingXL'
  | 'headingLarge'
  | 'headingMedium'
  | 'headingSmall'
  | 'headingXS'
  | 'displayLarge'
  | 'displayMedium'
  | 'displaySmall'
  | 'copyMedium'
  | 'copySmall'
  | 'copyXS';

interface BaseTextProps {
  variant?: TextVariant;
  semanticElement?: TextSemanticElement;
  styleVariant?: TextStyleVariant;
  children: React.ReactNode;
}

type StyledSystemTextProps = TypographyProps &
  Omit<ColorProps, 'bg' | 'backgroundColor' | 'opacity'>;

type TextProps = BaseTextProps & StyledSystemTextProps;

declare const Text: ({ variant, semanticElement, styleVariant, children, ...props }: TextProps) => JSX.Element;

type GridGridMode = 'wide' | 'condensed';

interface GridProps$1 extends BoxProps, AdditionalPropsInterface {
  /**
   * Just in case one needs to remove the outer padding provided.
   * Especially useful if using a `Grid` within a `Grid`.
   */
  noOuterGutter?: boolean;
  /**
   * Shows a debug set of styling for `Grid`, `Row`, and `Column` components
   * nested within the scope of a `Grid`.
   */
  debug?: boolean;
  /**
   * Defines the gutter sizing within a `Grid`'s context
   */
  gridMode?: GridGridMode;
}

declare const Grid: ({ debug, gridMode, ...props }: GridProps$1) => JSX.Element;

interface RowProps extends BoxProps, AdditionalPropsInterface {
  children?: React.ReactNode;
}

declare const Row: ({ children, ...props }: RowProps) => JSX.Element;

interface ColumnProps extends BoxProps, AdditionalPropsInterface {
  /**
   * This specifies the width of the column. Accepts:
   * - `number`: a number of columns to span in the 16-column breakpoints.
   * - 'auto': turns on automatic shrink/expand mode
   * - 'none': hides the column
   * - responsive object with the same options at each breakpoint
   */
  spans?: any;
  /**
   * The number of column spans to offset from the left
   */
  offsets?: any;
  children?: React.ReactNode;
}

declare const Column: ({ children, spans, offsets, ...props }: ColumnProps) => JSX.Element;

type ButtonVariant =
  | 'solidBlue'
  | 'solidAlt'
  | 'outlineBlue'
  | 'outlineAlt'
  | 'outlineInverse'
  | 'ghostBlue'
  | 'ghostAlt'
  | 'ghostInverse';

type ButtonSize = 'large' | 'medium' | 'small';

interface ButtonProps extends AdditionalPropsInterface {
  /**
   * The component accepts `children`, but internally the value will be passed
   * to `Text` component. The prop expects a `string`, not a React node
   */
  children?: string | string[];
  /**
   * Pass-through for the `onClick` functionality
   */
  onClick?: (...args: any[]) => any;
  /**
   * Tied directly to the purpose of the button
   */
  variant?: ButtonVariant;
  /**
   * Affects height and font size
   */
  size?: ButtonSize;
  /**
   * Whether or not the button should be disabled
   */
  disabled?: boolean;
}

declare const Button: ({ children, onClick, variant, size, disabled, ...props }: ButtonProps) => JSX.Element;

type ButtonGroupGroupRightAt = number | string;

interface ButtonGroupProps extends AdditionalPropsInterface {
  /**
   * Expects only `Button` components, and will not render anything else
   */
  children?: React.ReactNode;
  /**
   * Given the Button children as an array, the index to start grouping to the
   * right at. Buttons will group to the left by default.
   */
  groupRightAt?: ButtonGroupGroupRightAt;
}

declare const ButtonGroup: ({ children, groupRightAt, ...props }: ButtonGroupProps) => JSX.Element;

interface CheckboxInputProps extends AdditionalPropsInterface {
  /**
   * The controlled value of the input
   */
  checked: boolean;
  /**
   * Array of element `id`s, used by screen readers to provide additional context for the input
   */
  describedBy?: string[];
  /**
   * The id for the input
   */
  id?: string;
  /**
   * Use this prop to set initial focus to the input
   */
  isFocused?: boolean;
  /**
   * The label for the input
   */
  label: string;
  /**
   * Array of element `id`s, used by screen readers to provide essential context for the input
   */
  labelledBy?: string[];
  /**
   * Callback when the input is blurred
   */
  onBlur?: (...args: any[]) => any;
  /**
   * Occurs when the checkbox prop value on the input has been changed
   */
  onChange: (
    e: ReactChangeEvent<HTMLTextAreaElement | HTMLInputElement>,
  ) => void;
  /**
   * Callback when the input is focused
   */
  onFocus?: (...args: any[]) => any;
}

declare const CheckboxInput: ({ checked, describedBy: explicitDescribedBy, id: explicitId, isFocused, label, labelledBy: explicitLabelledBy, onBlur, onChange, onFocus, ...props }: CheckboxInputProps) => JSX.Element;

interface RadioInputProps extends AdditionalPropsInterface {
  /**
   * The controlled value of the input
   */
  checked: boolean;
  /**
   * Array of element `id`s, used by screen readers to provide additional context for the input
   */
  describedBy?: string[];
  /**
   * The id for the input
   */
  id?: string;
  /**
   * Use this prop to set initial focus to the input
   */
  isFocused?: boolean;
  /**
   * The label for the input
   */
  label: string;
  /**
   * Array of element `id`s, used by screen readers to provide essential context for the input
   */
  labelledBy?: string[];
  /**
   * Callback when the input is blurred
   */
  onBlur?: (...args: any[]) => any;
  /**
   * Occurs when the checkbox prop value on the input has been changed
   */
  onChange: (
    e: ReactChangeEvent<HTMLTextAreaElement | HTMLInputElement>,
  ) => void;
  /**
   * Callback when the input is focused
   */
  onFocus?: (...args: any[]) => any;
}

declare const RadioInput: ({ checked, describedBy: explicitDescribedBy, id: explicitId, isFocused, label, labelledBy: explicitLabelledBy, onBlur, onChange, onFocus, ...props }: RadioInputProps) => JSX.Element;

interface TextInputProps extends AdditionalPropsInterface {
  /**
   * Array of element `id`s, used by screen readers to provide additional context for the input
   */
  describedBy?: string[];
  /**
   * The error message will be displayed if provided, it triggers the error styling
   */
  error?: string | null;
  /**
   * Visually hides the label, but still allows access for screen readers
   */
  hideLabel?: boolean;
  /**
   * The id for the text input
   */
  id?: string;
  /**
   * Whether or not the input should be forced to be focused
   */
  isFocused?: boolean;
  /**
   * The label for the input
   */
  label: string;
  /**
   * Callback when the input is blurred
   */
  onBlur?: (...args: any[]) => any;
  /**
   * The handler for setting the value.
   * It expects only the input value in the handler, not the whole change event.
   */
  onChange: (...args: any[]) => any;
  /**
   * Callback when the input is focused
   */
  onFocus?: (...args: any[]) => any;
  /**
   * Placeholder text
   */
  placeholder?: string;
  /**
   * The controlled value of the input
   */
  value: string;
}

declare const TextInput: ({ describedBy: explicitDescribedBy, error, hideLabel, id: explicitId, isFocused, label, onBlur, onChange, onFocus, placeholder, value: inputValue, ...props }: TextInputProps) => JSX.Element;

interface CardProps extends BoxProps, AdditionalPropsInterface {
  children: React.ReactNode;
}

declare const Card: ({ children, ...props }: CardProps) => JSX.Element;

type SpinnerSize = 'small' | 'medium' | 'large';

interface SpinnerProps extends AdditionalPropsInterface {
  /**
   * How many milliseconds before the spinner starts spinning
   */
  delay?: number;
  /**
   * Which size the spinner should be
   */
  size?: SpinnerSize;
}

declare const Spinner: ({ delay, size, ...props }: SpinnerProps & AdditionalPropsInterface) => JSX.Element | null;

interface InputGroupProps extends BoxProps, AdditionalPropsInterface {
  /**
   * The label for the input group as a whole
   */
  label?: string;
  /**
   * Object of key/value pairs that will be applied as prop overrides to the inner
   * div that wraps the inputs. Defaults to have display=flex and a negative margin
   * that ensures the inputs are flush to the edge
   */
  childrenWrapperProps?: any;
  children?: React.ReactNode;
}

declare const InputGroup: ({ children, label, childrenWrapperProps, ...props }: InputGroupProps) => JSX.Element;

declare const theme: ThemeInterface;

export { AppProvider, Box, Button, ButtonGroup, Card, CheckboxInput, Column, Grid, InputGroup, RadioInput, Row, Spinner, Text, TextInput, theme };
